#' Plot cell type composition in each density level of cell of interest.
#'
#' @param spe A SpatialExperiment object.
#' @param coi A character vector of cell types of interest (COIs).
#' @param id A character. The name of the column of colData(spe)
#' containing the cell type identifiers. Set to cell_type by default.
#' @param level.name The column name that generated by cellAssign.
#' @param by.roi Logical. Default is FALSE, set to TRUE to allow
#' plotting by ROI.
#'
#' @return A ggplot object.
#' @export
#'
#' @examples
#'
#' data("xenium_bc_spe")
#' spe <- gridDensity(spe, coi = c("Breast cancer", "Fibroblasts"))
#' spe <- findROI(spe, coi = c("Breast cancer", "Fibroblasts"))
#' spe <- getContour(spe, coi = "Breast cancer")
#' spe <- allocateCells(spe)
#' plotCellCompo(spe, coi = "Breast cancer")
#' plotCellCompo(spe, coi = "Breast cancer", by.roi = TRUE)
#'
plotCellCompo <- function(spe, coi, id = "cell_type",
                          level.name = paste0(
                              janitor::make_clean_names(coi),
                              "_contour"
                          ), by.roi = FALSE) {
    dat <- as.data.frame(colData(spe))

    if (!(id %in% colnames(dat)) | !(level.name %in% colnames(dat))) {
        stop("Parameter id and level.name are expected to be in
        the column names of the colData of spe.")
    }

    if (isFALSE(by.roi)) {
        dat <- as.data.frame(colData(spe))[, c(id, level.name)]
    } else {
        if (!("roi" %in% colnames(dat))) {
            stop("Column roi not found, please run allocateCells
           with to.roi=TRUE first.")
        }
        dat <- as.data.frame(colData(spe))[, c(id, level.name, "roi")]
    }

    if (!(coi %in% dat$cell_type)) {
        stop("Parater coi is expected to be one of the
             cell types in the data.")
    }

    dat <- dat[dat$cell_type != coi, ]

    if (isFALSE(by.roi)) {
        toplot <- calc_proportions(dat, level.name, id)
    } else {
        dat <- dat[dat[["roi"]] != "no_roi", ]

        grouped_data <- split(dat, as.character(dat[["roi"]]))

        proportions_by_roi <- lapply(grouped_data, function(group) {
            calc_proportions(group, level.name, id, by.roi)
        })

        toplot <- do.call(rbind, proportions_by_roi)
    }

    col.p <- selectColor(length(unique(toplot[["cell_type"]])))

    p <- ggplot(toplot, aes(
        x = !!rlang::sym(level.name),
        y = Proportion,
        fill = cell_type
    )) +
        geom_bar(stat = "identity") +
        scale_fill_manual("Cell type", values = col.p) +
        ggtitle(paste0(
            "Cell type composition at each ", coi,
            " density level"
        )) +
        theme_classic()

    if (isFALSE(by.roi)) {
        return(p)
    } else {
        p <- p +
            facet_wrap(~roi)

        return(p)
    }
}

calc_proportions <- function(x, level.name, id, by.roi = FALSE) {
    cell_count <- table(x[[level.name]], x[[id]])
    cell_prop <- data.frame(prop.table(cell_count, margin = 1))
    colnames(cell_prop) <- c(level.name, id, "Proportion")
    cell_prop <- cell_prop[!is.na(cell_prop$Proportion), ]
    if (isTRUE(by.roi)) {
        cell_prop$roi <- unique(x[["roi"]])
    }
    return(cell_prop)
}

utils::globalVariables(c("Proportion", "cell_type"))
